﻿--[[
	MrtWoo_AntiSpam: Detects and blocks russian commercial spam and flud
	Copyright (C) 2010 Pavel Dudkovsky (mrtime@era.by)
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
]]

local MrtWoo = MrtWoo;
local AntiSpam = MrtWoo:NewModule("AntiSpam", "AceEvent-3.0");
AntiSpam.Version = "r28";

local opts = {
	as_options = {
		type="group",
		name="AntiSpam",
		args={
			silent = {
				type = "toggle",
				name = "Тихий режим",
				desc = "Не отображать сообщений при блокировке спама",
				get = function() return MrtWoo.db.profile.modules.as_silent end,
				set = function() MrtWoo.db.profile.modules.as_silent = not MrtWoo.db.profile.modules.as_silent end,
				order=1,
			},
			
			spam_rate = {
				type = "range",
				max = 500,
				min = 1,
				bigStep = 1,
				step = 1,
				name = "Максимальный рейтинг сообщения после которого оно будет заблокировано",
				desc = "После получения сообщения оно проверяется на наличие спама и ему присваиваеться рейтинг, обычно нормальное сообщение не содержащее спам имеет рейтинг 0-75...Если рейтинг сообщения превысит данный порог блокировки, то сообщение будет расценено как спам и все сообщения от этого пользователя будут заблокированны.",
				get = function() return MrtWoo.db.profile.modules.as_spamrate end,
				set = function(info, input) MrtWoo.db.profile.modules.as_spamrate = input end,
				order=2,
			},
			
			spam_additional_checks = {
				type = "toggle",
				name = "Выполнять дополнительные проверки?",
				desc = "Часто бывает так что проверки только сообщения не достаточно... поэтому иногда нужно проверить уровень персонажа, его ник, ...",
				get = function() return MrtWoo.db.profile.modules.as_spam_additional_checks end,
				set = function() MrtWoo.db.profile.modules.as_spam_additional_checks = not MrtWoo.db.profile.modules.as_spam_additional_checks end,
				order=3,
			},
						
			spam_rate_needed_for_additional_checks = {
				type = "range",
				max = 500,
				min = 1,
				bigStep = 1,
				step = 1,
				name = "Рейтинг сообщения необходимый для проведения дополнительных проверок",
				desc = "Значение данного параметра должно быть меньше значения рейтинга блокировки, иначе дополнительные проверки проводиться не будут",
				get = function() return MrtWoo.db.profile.modules.as_spamrate_needed_for_additional_checks end,
				set = function(info, input) MrtWoo.db.profile.modules.as_spamrate_needed_for_additional_checks = input end,
				order=4,
			},
						
			spam_maxlvl = {
				type = "range",
				max = 85,
				min = 1,
				bigStep = 1,
				step = 1,
				name = "Максимальный не надежный уровень игрока",
				desc = "Часто спамеры не качают персов выше 15 левела...",
				get = function() return MrtWoo.db.profile.modules.as_spam_maxlvl end,
				set = function(info, input) MrtWoo.db.profile.modules.as_spam_maxlvl = input end,
				order=5,
			},

			spam_ratelvl = {
				type = "range",
				max = 500,
				min = 1,
				bigStep = 1,
				step = 1,
				name = "Добавочный рейтинг сообщения если автор не надежного уровня",
				desc = "Укажите добавочный рейтинг сообщения, он будет добавлен к рейтингу сообщения который был получен после проверки содержания... если суммарный рейтинг сообщеня превысит порог блокировки, то сообщение будет расценено как спам и все сообщения от данного пользователя будут заблокированны",
				get = function() return MrtWoo.db.profile.modules.as_spam_ratelvl end,
				set = function(info, input) MrtWoo.db.profile.modules.as_spam_ratelvl = input end,
				order=6,
			},
						
			gc_wait_time = {
				type = "range",
				max = 600,
				min = 1,
				bigStep = 1,
				step = 1,
				name = "Интервал сборки мусора (сек)",
				desc = "Значение данного параметра влияет на то как часто будут происходить попытки удалить не нужные данные из памяти",
				get = function() return MrtWoo.db.profile.modules.as_gc_wait_time end,
				set = function(info, input) MrtWoo.db.profile.modules.as_gc_wait_time = input end,
				order=7,
			},

			gc_message_lifetime = {
				type = "range",
				max = 600,
				min = 1,
				bigStep = 1,
				step = 1,
				name = "Время жизни сообщения в кеше после проверки (сек)",
				desc = "Значение данного параметра влияет на то сколько сообщение будет храниться в очереди, если есть проблемы с выводом сообщений попробуйте увеличить значение данного параметра",
				get = function() return MrtWoo.db.profile.modules.as_gc_message_lifetime end,
				set = function(info, input) MrtWoo.db.profile.modules.as_gc_message_lifetime = input end,
				order=8,
			},
				
			gc_checkinfo_lifetime = {
				type = "range",
				max = 3600,
				min = 1,
				bigStep = 1,
				step = 1,
				name = "Время жизни информации о статусе последней проверки сообщения пользователя в кеше (сек)",
				desc = "Значение данного параметра влияет на то как часто будут происходить повторные проверки на спам и флуд",
				get = function() return MrtWoo.db.profile.modules.as_gc_checkinfo_lifetime end,
				set = function(info, input) MrtWoo.db.profile.modules.as_gc_checkinfo_lifetime = input end,
				order=9,
			},
						
			debug = {
				type = "toggle",
				name = "Отладка",
				desc = "Имеет смысл вкоючать при поиске ошибок(выводит дополнительную информацию во время работы аддона)",
				get = function() return MrtWoo.db.profile.modules.as_debug end,
				set = function() MrtWoo.db.profile.modules.as_debug = not MrtWoo.db.profile.modules.as_debug end,
				order=10,
			},			
		}
	}
};

function AntiSpam:SetDefaults()
	MrtWoo.defaults.profile.modules.as_silent = true;
	MrtWoo.defaults.profile.modules.as_debug = false;
	MrtWoo.defaults.profile.modules.as_spam_additional_checks = true;
	MrtWoo.defaults.profile.modules.as_spam_maxlvl = 15;
	MrtWoo.defaults.profile.modules.as_spam_ratelvl = 50;
	MrtWoo.defaults.profile.modules.as_spamrate = 100;
	MrtWoo.defaults.profile.modules.as_spamrate_needed_for_additional_checks = 50;
	MrtWoo.defaults.profile.modules.as_gc_wait_time = 150;
	MrtWoo.defaults.profile.modules.as_gc_message_lifetime = 60;
	MrtWoo.defaults.profile.modules.as_gc_checkinfo_lifetime = 600;
end;

function AntiSpam:OnInitialize()
	self.SetDefaults();
	table.insert(MrtWoo.options.plugins, opts);
	
	MrtWoo:RegisterChatCommand("mrtwoo_show_smi", AntiSpam.ShowSpamMonInfo);
	MrtWoo:RegisterChatCommand("mrtwoo_test", AntiSpam.AsTest);
		
	self.Monitor = MrtWoo_AntiSpamMonitor:new();
	
	self:RegisterMessage('MRTWOO_ASMON_PROCESSED_MESSAGE', AntiSpam.SendProcessedMessage);
end;

function AntiSpam:ShowSpamMonInfo(input)
	-- AntiSpam.MsgsQueue:Show();
	AntiSpam.Monitor:ShowSMI();
end;


function AntiSpam:AsTest(input)
	local test = "Продам голд по 60р aся 111-111-111";
	local bl = MrtWoo_AntiSpamBlockList:new();
	local rate = bl:RateTradeMessage(test);
	
	MrtWoo:Print('TEST:', rate);
end;

function AntiSpam:OnEnable()
	ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", AntiSpam.MessagesEventFilter);
end;

function AntiSpam:OnDisable()
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_CHANNEL", AntiSpam.MessagesEventFilter);
end;


function AntiSpam:SendProcessedMessage(msgType, ...)
	local old_self = self;

	for i=1,NUM_CHAT_WINDOWS do			
		local chatWin = getglobal("ChatFrame"..i);
		if chatWin then			
			local channels = {GetChatWindowMessages(chatWin:GetID())}
			for x=1,#channels do
				if channels[x] == msgType then					

					self = chatWin;

					self.mrtwoo_spm = true;
					ChatFrame_OnEvent(self, ...);
					self.mrtwoo_spm = false;

					self = old_self;

					break;
				end;
			end;
		end;
	end;
end;

function AntiSpam:MessagesEventFilter(event, ...)
	local msg  = select(1, ...);
	local author = select(2, ...);
		
	if self.mrtwoo_spm or (string.len(msg) == 0) or (string.len(author) == 0) then
		return false, ...;
	end;
	
	local msgId = select(11, ...);	
		
	AntiSpam.Monitor:AddMessageToQueue(msgId, event, ...);
		
	return true, ...;
end;
